# Generated by roxygen2: do not edit by hand

export(aux.fisch)
export(lsolve.bicg)
export(lsolve.bicgstab)
export(lsolve.cg)
export(lsolve.cgs)
export(lsolve.cheby)
export(lsolve.gmres)
export(lsolve.gs)
export(lsolve.jacobi)
export(lsolve.qmr)
export(lsolve.sor)
export(lsolve.ssor)
import(Matrix)
import(Rdpack)
importFrom(Rcpp,evalCpp)
importFrom(stats,rnorm)
importFrom(utils,packageVersion)
useDynLib(Rlinsolve, .registration=TRUE)
