# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @keywords internal
#' @noRd
linsolve.bicg.single <- function(A, b, xinit, reltol, maxiter, M) {
    .Call(`_Rlinsolve_single_bicg`, A, b, xinit, reltol, maxiter, M)
}

#' @keywords internal
#' @noRd
linsolve.bicg.single.sparse <- function(A, b, xinit, reltol, maxiter, M) {
    .Call(`_Rlinsolve_single_bicg_sparse`, A, b, xinit, reltol, maxiter, M)
}

#' @keywords internal
#' @noRd
linsolve.bicgstab.single <- function(A, b, xinit, reltol, maxiter, M) {
    .Call(`_Rlinsolve_single_bicgstab`, A, b, xinit, reltol, maxiter, M)
}

#' @keywords internal
#' @noRd
linsolve.bicgstab.single.sparse <- function(A, b, xinit, reltol, maxiter, M) {
    .Call(`_Rlinsolve_single_bicgstab_sparse`, A, b, xinit, reltol, maxiter, M)
}

#' @keywords internal
#' @noRd
linsolve.cg.single <- function(A, b, xinit, reltol, maxiter, M) {
    .Call(`_Rlinsolve_single_cg`, A, b, xinit, reltol, maxiter, M)
}

#' @keywords internal
#' @noRd
linsolve.cg.single.sparse <- function(A, b, xinit, reltol, maxiter, M) {
    .Call(`_Rlinsolve_single_cg_sparse`, A, b, xinit, reltol, maxiter, M)
}

#' @keywords internal
#' @noRd
linsolve.cgs.single <- function(A, b, xinit, reltol, maxiter, M) {
    .Call(`_Rlinsolve_single_cgs`, A, b, xinit, reltol, maxiter, M)
}

#' @keywords internal
#' @noRd
linsolve.cgs.single.sparse <- function(A, b, xinit, reltol, maxiter, M) {
    .Call(`_Rlinsolve_single_cgs_sparse`, A, b, xinit, reltol, maxiter, M)
}

#' @keywords internal
#' @noRd
linsolve.cheby.single <- function(A, b, xinit, reltol, maxiter, M, eigmax, eigmin) {
    .Call(`_Rlinsolve_single_cheby`, A, b, xinit, reltol, maxiter, M, eigmax, eigmin)
}

#' @keywords internal
#' @noRd
linsolve.cheby.single.sparse <- function(A, b, xinit, reltol, maxiter, M, eigmax, eigmin) {
    .Call(`_Rlinsolve_single_cheby_sparse`, A, b, xinit, reltol, maxiter, M, eigmax, eigmin)
}

#' @keywords internal
#' @noRd
linsolve.gmres.single <- function(A, b, xinit, reltol, maxiter, M, restrt) {
    .Call(`_Rlinsolve_single_gmres`, A, b, xinit, reltol, maxiter, M, restrt)
}

#' @keywords internal
#' @noRd
linsolve.gmres.single.sparse <- function(A, b, xinit, reltol, maxiter, M, restrt) {
    .Call(`_Rlinsolve_single_gmres_sparse`, A, b, xinit, reltol, maxiter, M, restrt)
}

#' @keywords internal
#' @noRd
linsolve.qmr.single <- function(A, b, xinit, reltol, maxiter, M) {
    .Call(`_Rlinsolve_single_qmr`, A, b, xinit, reltol, maxiter, M)
}

#' @keywords internal
#' @noRd
linsolve.qmr.single.sparse <- function(A, b, xinit, reltol, maxiter, M, M1, M2) {
    .Call(`_Rlinsolve_single_qmr_sparse`, A, b, xinit, reltol, maxiter, M, M1, M2)
}

#' @keywords internal
#' @noRd
linsolve.gs.single <- function(A, b, xinit, reltol, maxiter, dflagval) {
    .Call(`_Rlinsolve_single_gs`, A, b, xinit, reltol, maxiter, dflagval)
}

#' @keywords internal
#' @noRd
linsolve.gs.single.sparse <- function(A, b, xinit, reltol, maxiter, dflagval) {
    .Call(`_Rlinsolve_single_gs_sparse`, A, b, xinit, reltol, maxiter, dflagval)
}

#' @keywords internal
#' @noRd
linsolve.jacobi.single <- function(A, b, xinit, reltol, maxiter, weight) {
    .Call(`_Rlinsolve_single_jacobi`, A, b, xinit, reltol, maxiter, weight)
}

#' @keywords internal
#' @noRd
linsolve.jacobi.single.sparse <- function(A, b, xinit, reltol, maxiter, weight) {
    .Call(`_Rlinsolve_single_jacobi_sparse`, A, b, xinit, reltol, maxiter, weight)
}

#' @keywords internal
#' @noRd
linsolve.sor.single <- function(A, b, xinit, reltol, maxiter, w) {
    .Call(`_Rlinsolve_single_sor`, A, b, xinit, reltol, maxiter, w)
}

#' @keywords internal
#' @noRd
linsolve.sor.single.sparse <- function(A, b, xinit, reltol, maxiter, w) {
    .Call(`_Rlinsolve_single_sor_sparse`, A, b, xinit, reltol, maxiter, w)
}

#' @keywords internal
#' @noRd
linsolve.ssor.single <- function(A, b, xinit, reltol, maxiter, w) {
    .Call(`_Rlinsolve_single_ssor`, A, b, xinit, reltol, maxiter, w)
}

#' @keywords internal
#' @noRd
linsolve.ssor.single.sparse <- function(A, b, xinit, reltol, maxiter, w) {
    .Call(`_Rlinsolve_single_ssor_sparse`, A, b, xinit, reltol, maxiter, w)
}

